"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __exportStar = (this && this.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AppGraphQl = void 0;
const graphql_1 = require("graphql");
const schema_1 = require("@graphql-tools/schema");
const apollo_server_core_1 = require("apollo-server-core");
const apollo_server_express_1 = require("apollo-server-express");
const cm_libs_context_aware_logger_1 = require("cm-libs-context-aware-logger");
const cm_libs_context_aware_error_1 = require("cm-libs-context-aware-error");
const common_1 = require("./common");
const app_base_1 = require("./app-base");
const app_graphql_directive_1 = require("./app-graphql-directive");
__exportStar(require("@graphql-tools/schema"), exports);
__exportStar(require("apollo-server-core"), exports);
__exportStar(require("apollo-server-express"), exports);
__exportStar(require("apollo-server-plugin-base"), exports);
class AppGraphQl extends app_base_1.AppBase {
    async init(schema, resolvers, origins, overrides, rateLimitHandler) {
        await super.init(origins, rateLimitHandler, overrides);
        const transformedSchema = (0, app_graphql_directive_1.applyDirectiveTransformers)((0, schema_1.makeExecutableSchema)({
            typeDefs: [app_graphql_directive_1.directiveTypeDefs, schema],
            resolvers
        }));
        // Keep the full schema
        this._schema = (0, graphql_1.print)(schema);
        const serverConfig = {
            schema: transformedSchema,
            context: async ({ req }) => {
                return this.context(req);
            },
            formatError: this.formatError,
            csrfPrevention: true,
            cache: 'bounded',
            ...overrides
        };
        if (this.devMode) {
            serverConfig.plugins = [(0, apollo_server_core_1.ApolloServerPluginLandingPageGraphQLPlayground)()];
        }
        const server = new apollo_server_express_1.ApolloServer(serverConfig);
        await server.start();
        this.app.use(server.getMiddleware({ path: overrides?.path || '/graphql' }));
        return this;
    }
    context(req) {
        const ip = (0, common_1.getIp)(req);
        req['logExcluded'] = false;
        (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.NORMAL, () => `REQ: ${req.method} ${req.baseUrl} HTTP/${req.httpVersion} @${ip}`);
        return {
            req,
            ip: (0, common_1.getIp)(req),
            accessInfo: (0, common_1.getAuthToken)(req)?.info
        };
    }
    formatError(error) {
        const err = (0, cm_libs_context_aware_error_1.describeError)(error?.originalError || error?.extensions);
        if (error?.path && 0 < error?.path?.length) {
            err['path'] = error.path[0];
        }
        (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.ERROR, () => `GraphQL ERROR @ ${err['path'] || '<PATH>'} - ${err.code}: ${err.reason || err.message || cm_libs_context_aware_error_1.errors.UNKNOWN}`);
        const ret = {
            message: err.message,
            extensions: {
                code: err.code,
                details: error.originalError?.details
            }
        };
        return ret;
    }
    get schema() {
        return this._schema;
    }
}
exports.AppGraphQl = AppGraphQl;
//# sourceMappingURL=app-graphql.js.map