"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AuthBearer = void 0;
const graphql_1 = require("graphql");
const apollo_server_core_1 = require("apollo-server-core");
const cm_libs_context_aware_error_1 = require("cm-libs-context-aware-error");
const intercall_1 = require("../intercall");
const auth_type_1 = require("./auth-type");
class AuthBearer {
    constructor(url) {
        this.identityUrl = url;
    }
    static init(url) {
        this.self = new AuthBearer(url);
    }
    static async updateAuthToken(authToken) {
        if (undefined === authToken?.info?.token) {
            (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.UNAUTHORISED, 'Id token is missing');
        }
        const query = (0, apollo_server_core_1.gql) `query PaidChain{
            verifyIdToken(idToken:"${authToken.info.token.replace('Bearer ', '')}")
        }`;
        const idToken = await (0, intercall_1.callGraphQl)(this.instance.identityUrl, (0, graphql_1.print)(query), 'verifyIdToken');
        authToken.type = auth_type_1.AuthTokenType.Bearer;
        authToken.grant = true;
        authToken.info.auth = idToken;
        authToken.info.requestedBy = idToken.userId;
        return authToken;
    }
    static get instance() {
        return this.self || (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.NOT_INITIALIZED, 'AuthBearer initialization is required');
    }
}
exports.AuthBearer = AuthBearer;
//# sourceMappingURL=auth-bearer.js.map