"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.hasAbility = exports.isAuthenticated = exports.auth = void 0;
const ability_1 = require("@casl/ability");
const cm_libs_context_aware_error_1 = require("cm-libs-context-aware-error");
const http_1 = require("../http");
const auth_type_1 = require("./auth-type");
const auth_bearer_1 = require("./auth-bearer");
const auth_basic_1 = require("./auth-basic");
async function auth(req, authDirective) {
    const authToken = (0, http_1.getAuthToken)(req);
    if (authToken?.info?.token) {
        if (undefined === authToken.grant) {
            switch (authDirective.type) {
                case auth_type_1.AuthTokenType.Bearer: {
                    const _authToken = await auth_bearer_1.AuthBearer.updateAuthToken(authToken);
                    // Set the auth token into req object
                    (0, http_1.setAuthToken)(req, _authToken);
                    return _authToken;
                }
                case auth_type_1.AuthTokenType.Basic: {
                    const loginInfo = authToken.info.authMetadata;
                    const _authToken = await auth_basic_1.AuthBasic.updateAuthToken(authToken, loginInfo?.username, loginInfo?.password, authToken.info.headers?.channel);
                    // Set the auth token into req object
                    (0, http_1.setAuthToken)(req, _authToken);
                    return _authToken;
                }
            }
        }
        if (true === authToken.grant) {
            return authToken;
        }
    }
    (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.UNAUTHORISED, 'Unauthorized access', { hasToken: undefined !== authToken?.info?.token, spec: authDirective });
}
exports.auth = auth;
function isAuthenticated(authToken) {
    return true === authToken?.grant;
}
exports.isAuthenticated = isAuthenticated;
function hasAbility(authToken, spec) {
    if (!isAuthenticated(authToken) || !spec) {
        (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.UNAUTHORISED, 'Unauthorized access', { authToken, spec });
    }
    const { action, subject } = spec;
    const ability = (0, ability_1.defineAbility)((_can, cannot) => {
        cannot('manage', 'all');
    });
    const rules = authToken.info?.auth?.rules;
    if (rules) {
        // Inspect ability by rules
        ability.update(JSON.parse(rules));
    }
    if (!ability.can(action, subject)) {
        ability_1.ForbiddenError.from(ability).throwUnlessCan(action, subject);
    }
}
exports.hasAbility = hasAbility;
//# sourceMappingURL=auth.js.map