'use strict';
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __exportStar = (this && this.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DomainEventConsumer = void 0;
const cm_libs_context_aware_error_1 = require("cm-libs-context-aware-error");
const cm_libs_kafka_1 = require("cm-libs-kafka");
const cm_libs_utils_1 = require("cm-libs-utils");
const domain_event_1 = require("./domain-event");
__exportStar(require("cm-libs-kafka"), exports);
class DomainEventConsumer {
    constructor(config, groupId, topic, heartbeatThreshold) {
        this.config = config;
        this.groupId = groupId;
        this.topic = topic;
        this.heartbeatThreshold = heartbeatThreshold;
        this.domainEvent = new domain_event_1.DomainEvent();
    }
    async term() {
        if (this._consumer) {
            await this._consumer.term();
            this._consumer = undefined;
        }
    }
    async subscribe(handler, exceptionHandler) {
        if (this._consumer) {
            (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.SUBSCRIBE_ERROR, 'Duplicated domain event subscription', { topic: this.topic });
        }
        this._consumer = await cm_libs_kafka_1.KafkaConsumer.init(this.config, this.groupId, this.topic, async (payload) => {
            let message;
            try {
                message = await this.domainEvent.decode(payload.message?.value);
            }
            catch (error) {
                if (!exceptionHandler) {
                    throw error;
                }
                return await exceptionHandler(error, undefined, payload);
            }
            try {
                return await handler(message, payload);
            }
            catch (error) {
                if (!exceptionHandler) {
                    throw error;
                }
                return await exceptionHandler(error, message, payload);
            }
        }, { fromBeginning: false /** dont consume all existing message for new groupId */ });
    }
    async unsubscribe() {
        if (this._consumer) {
            await this._consumer.term();
            this._consumer = undefined;
        }
    }
    async getOffset(partition) {
        return cm_libs_kafka_1.KafkaConsumer.getOffset(this.config, this.groupId, this.topic, partition);
    }
    get pulseTimestamp() {
        if (this._consumer) {
            return this._consumer.pulseTimestamp;
        }
        return 0;
    }
    get consumer() {
        if (this._consumer) {
            return this._consumer.consumer;
        }
    }
    get ready() {
        if (this._consumer) {
            if (0 < this.heartbeatThreshold) {
                return this.heartbeatThreshold >= (0, cm_libs_utils_1.timeElapsed)(this._consumer.pulseTimestamp);
            }
            return 0 < this._consumer.pulseTimestamp;
        }
        return false;
    }
}
exports.DomainEventConsumer = DomainEventConsumer;
//# sourceMappingURL=domain-event-consumer.js.map