'use strict';
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __exportStar = (this && this.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DomainEventProducer = void 0;
const cm_libs_kafka_1 = require("cm-libs-kafka");
const cm_libs_utils_1 = require("cm-libs-utils");
const nanoid_1 = require("nanoid");
const domain_event_1 = require("./domain-event");
__exportStar(require("cm-libs-kafka"), exports);
class DomainEventProducer {
    constructor(kafkaConfig, topics, schema, metadataSchema) {
        this.kafkaConfig = kafkaConfig;
        this.topics = topics || [];
        this.domainEvent = new domain_event_1.DomainEvent(schema, metadataSchema);
    }
    async term() {
        if (this.producer) {
            await this.producer.term();
            this.producer = undefined;
        }
    }
    async publish(message, topic) {
        message.eventType = message.eventType || null;
        message.eventId = message.eventId || (0, nanoid_1.nanoid)();
        message.correlationId = message.correlationId || (0, nanoid_1.nanoid)();
        message.timestamp = message.timestamp || (0, cm_libs_utils_1.timeNow)();
        const encodedMessage = await this.domainEvent.encode(message);
        const producer = await this.getProducer();
        const topics = topic ? this.topics.concat([topic]) : this.topics;
        if (1 === topics.length) {
            const result = await producer.send(topics[0], encodedMessage, message.eventId);
            return [result];
        }
        const batch = {
            topicMessages: []
        };
        for (const topic of topics) {
            batch.topicMessages.push({
                topic,
                messages: [
                    {
                        value: encodedMessage,
                        key: message.eventId
                    }
                ]
            });
        }
        return producer.sendBatch(batch);
    }
    async getProducer() {
        if (!this.producer) {
            this.producer = await cm_libs_kafka_1.KafkaProducer.init(this.kafkaConfig);
        }
        return this.producer;
    }
    get ready() {
        if (this.producer) {
            return this.producer.ready;
        }
        return true;
    }
    get schemaRegistryId() {
        return this.domainEvent.schemaRegistryId;
    }
    get metadataSchemaRegistryId() {
        return this.domainEvent.metadataSchemaRegistryId;
    }
}
exports.DomainEventProducer = DomainEventProducer;
//# sourceMappingURL=domain-event-producer.js.map