'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.DomainEvent = exports.DomainEventMessage = void 0;
const cm_libs_utils_1 = require("cm-libs-utils");
const cm_schema_event_1 = require("cm-schema-event");
Object.defineProperty(exports, "DomainEventMessage", { enumerable: true, get: function () { return cm_schema_event_1.DomainEventMessage; } });
const domain_schema_1 = require("./domain-schema");
const domain_event_codec_1 = require("./domain-event-codec");
const domain_event_cipher_1 = require("./domain-event-cipher");
class DomainEvent {
    constructor(schema, metadataSchema) {
        this.codec = new domain_event_codec_1.DomainEventCodec();
        this.schema = new domain_schema_1.DomainSchema(schema);
        this.metadataSchema = new domain_schema_1.DomainSchema(metadataSchema);
    }
    async encode({ eventType, eventId, correlationId, correlationTopic, timestamp, body, metadata }) {
        const eventMessage = {
            eventType,
            eventId,
            correlationId,
            correlationTopic,
            timestamp: undefined === timestamp ? (0, cm_libs_utils_1.timeNow)() : timestamp,
            body: await this.schema.encode(body),
            metadata: await this.metadataSchema.encode(metadata)
        };
        return this.codec.encode(eventMessage);
    }
    async decode(encodedMessage) {
        const { eventType, eventId, correlationId, correlationTopic, timestamp, body, metadata } = await this.codec.decode(encodedMessage);
        const message = {
            eventType,
            eventId,
            correlationId,
            correlationTopic,
            timestamp,
            body: await this.schema.decode(body),
            metadata: await this.metadataSchema.decode(metadata)
        };
        return message;
    }
    static init(bdkSet, ksn) {
        domain_event_cipher_1.DomainEventCipher.init(bdkSet, ksn);
    }
    get schemaRegistryId() {
        return this.schema.registryId;
    }
    get metadataSchemaRegistryId() {
        return this.metadataSchema.registryId;
    }
}
exports.DomainEvent = DomainEvent;
//# sourceMappingURL=domain-event.js.map