"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ErrorHandler = void 0;
const apollo_server_core_1 = require("apollo-server-core");
const cm_libs_context_aware_error_1 = require("cm-libs-context-aware-error");
const cm_libs_utils_1 = require("cm-libs-utils");
const intercall_1 = require("../intercall");
class ErrorHandler {
    constructor(url) {
        this.errorUrl = url;
    }
    static init(url) {
        this.self = new ErrorHandler(url);
    }
    static async translate(error, statusCode, localeCode) {
        const { code, message } = (0, cm_libs_context_aware_error_1.describeError)(error);
        // Override status code by error specified status code
        if (error?.statusCode) {
            statusCode = (0, cm_libs_utils_1.checkFormat)((0, cm_libs_utils_1.allowNullOrEmpty)(cm_libs_utils_1.checkNumber), error?.statusCode);
        }
        // Assign specific status code, otherwise respect the status code that caller specified
        if (!statusCode || 200 === statusCode) {
            // Override status code that based on the error code
            /* istanbul ignore next */
            switch (code) {
                case cm_libs_context_aware_error_1.errors.UNAUTHORISED:
                case cm_libs_context_aware_error_1.errors.EXPIRED:
                    statusCode = 401;
                    break;
                case cm_libs_context_aware_error_1.errors.FORBIDDEN:
                    statusCode = 403;
                    break;
                case 'TooManyRequests':
                    statusCode = 429;
                    break;
                default:
                    statusCode = 500;
            }
        }
        const translatedErrorMessage = this.isEnabled ? await this.instance.translateErrorMessage(code, localeCode) : message;
        return {
            statusCode,
            code,
            message: translatedErrorMessage
        };
    }
    async translateErrorMessage(code, localeCode = 'en_US') {
        const query = (0, apollo_server_core_1.gql) `query PaidChain{
            readErrorMessage(code:"${code}",localeCode:"${localeCode}")
            { reason }
        }`;
        try {
            const { reason } = await (0, intercall_1.callGraphQl)(this.errorUrl, query, 'readErrorMessage', { 'x-by-pass': true });
            return reason;
        }
        catch (error) {
            // Ignore
        }
    }
    static get instance() {
        return this.self || (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.NOT_INITIALIZED, 'ErrorHandler initialization is required');
    }
    static get isEnabled() {
        return undefined !== this.self;
    }
}
exports.ErrorHandler = ErrorHandler;
//# sourceMappingURL=error.js.map