"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.dumpResErrorMiddleware = exports.dumpResMiddleware = exports.dumpReqMiddleware = void 0;
const cm_libs_context_aware_error_1 = require("cm-libs-context-aware-error");
const cm_libs_context_aware_logger_1 = require("cm-libs-context-aware-logger");
const cm_libs_utils_1 = require("cm-libs-utils");
function dumpReqMiddleware(req, res, next) {
    (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.CRITICAL, () => {
        const headers = JSON.stringify(req.headers, null, 2);
        const body = JSON.stringify(req.body, null, 2);
        return `TRACE REQ: ${req.method} ${req.url}\nHEADERS: ${headers}\nBODY: ${body}`;
    });
    return next();
}
exports.dumpReqMiddleware = dumpReqMiddleware;
function dumpResMiddleware(req, res, next) {
    return dumpResErrorMiddleware(undefined, req, res, next);
}
exports.dumpResMiddleware = dumpResMiddleware;
function dumpResErrorMiddleware(error, req, res, next) {
    const resWrite = res.write;
    const resEnd = res.end;
    const url = req.url;
    const bodyChunks = [];
    res.write = (...args) => {
        bodyChunks.push(Buffer.from(args[0]));
        return resWrite.apply(res, args);
    };
    res.end = (...args) => {
        if (args[0]) {
            bodyChunks.push(Buffer.from(args[0]));
        }
        let body = 0 < bodyChunks.length ? Buffer.concat(bodyChunks).toString('utf8') : '';
        try {
            if (body && res.getHeader('content-type')?.toString().includes('application/json')) {
                const json = JSON.parse(body);
                body = JSON.stringify(json, null, 2);
            }
        }
        catch (error) {
            // Ignore
        }
        const headers = {};
        res.getHeaderNames()?.map((value) => {
            headers[value] = String(res.getHeader(value));
        });
        const { reason } = (0, cm_libs_context_aware_error_1.describeError)(error);
        const elapsed = req['start'] ? (0, cm_libs_utils_1.timeElapsed)(req['start']).toString() : '-';
        (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.CRITICAL, () => {
            return `TRACE RES: ${req.method} ${url} ${res.statusCode}, elapsed: ${elapsed + 'ms'}\nHEADERS: ${JSON.stringify(headers, null, 2)}\nBODY: ${body}${error ? '\n' + reason : ''}`;
        });
        return resEnd.apply(res, args);
    };
    return next();
}
exports.dumpResErrorMiddleware = dumpResErrorMiddleware;
//# sourceMappingURL=http-debug.js.map