"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.callRest = exports.callGraphQl = void 0;
const axios_1 = __importDefault(require("axios"));
const cm_libs_context_aware_error_1 = require("cm-libs-context-aware-error");
const cm_libs_context_aware_logger_1 = require("cm-libs-context-aware-logger");
const cm_libs_utils_1 = require("cm-libs-utils");
async function callGraphQl(url, query, fn, headers) {
    try {
        (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.DEBUG, () => `callGraphQl ${url} (x-channel: ${headers?.['x-channel']}) - ${fn}`);
        const config = defaultConfig({ headers });
        const { data: result } = await axios_1.default.post(url, { query }, config);
        const data = result?.data?.[fn];
        (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.INFO, () => `callGraphQl ${url} (x-channel: ${headers?.['x-channel']}) - ${fn}: ${data}`);
        if (!result || result.errors) {
            (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.INTERCALL_ERROR, 'callGraphQl ERROR', { url, query, fn, error: result?.errors[0] }, false);
        }
        return data;
    }
    catch (error) {
        (0, cm_libs_context_aware_error_1.throwFromError)(error, cm_libs_context_aware_error_1.errors.INTERCALL_ERROR, { url, query, fn, error: { ...error } });
    }
}
exports.callGraphQl = callGraphQl;
async function callRest(url, method, params, headers) {
    try {
        (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.DEBUG, () => `callRest ${method} ${url} (x-channel: ${headers?.['x-channel']})`);
        const config = defaultConfig({ url, method, headers, data: params });
        const { data: result } = await (0, axios_1.default)(config);
        const data = result?.data;
        (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.INFO, () => `callRest ${method} ${url} (x-channel: ${headers?.['x-channel']}): ${data}`);
        if (!result || result.errors) {
            (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.INTERCALL_ERROR, 'callRest ERROR', { url, data: params, method, error: result?.errors[0] }, false);
        }
        return data;
    }
    catch (error) {
        (0, cm_libs_context_aware_error_1.throwFromError)(error, cm_libs_context_aware_error_1.errors.INTERCALL_ERROR, { url, data: params, method, error: { ...error } });
    }
}
exports.callRest = callRest;
function defaultConfig(overrides) {
    const config = {
        maxBodyLength: Infinity,
        maxContentLength: Infinity,
        ...overrides
    };
    if ((0, cm_libs_utils_1.isUndefinedOrNullOrEmpty)(config.timeout)) {
        config.timeout = 120000; // Default timeout is 120 seconds
    }
    return config;
}
//# sourceMappingURL=intercall.js.map