import { nanoid } from 'nanoid';
export { nanoid };
export interface IPagination {
    pageSize?: number;
    page?: number;
}
export declare type BaseBrowse = {
    pagination?: IPagination;
    sort?: any;
    speedMode?: boolean;
};
declare enum RecordStatus {
    ACTIVE = "ACTIVE",
    INACTIVE = "INACTIVE"
}
export declare type BaseCreateInput = {
    uuid: string;
    createdAt: Date | string;
    createdBy: string;
    updatedAt: Date | string;
    updatedBy: string;
    remark?: string | null;
};
declare type BaseWhereInput = {
    status: RecordStatus;
};
declare type Dict = {
    [k: string]: any;
};
declare type DictWithId = {
    id?: number;
    [k: string]: any;
};
declare type SelectWithId = {
    id?: boolean;
    [k: string]: any;
};
interface BasePrismaClient {
    findMany: (arg: {
        select?: SelectWithId | null;
        include?: Dict | null;
        where?: Dict;
        orderBy?: unknown;
        cursor?: Dict;
        take?: number;
        skip?: number;
        distinct?: unknown;
    }) => any;
    create: (arg: {
        select?: SelectWithId | null;
        include?: Dict | null;
        data: any;
    }) => any;
    update: (arg: {
        select?: SelectWithId | null;
        include?: Dict | null;
        data: any;
        where: DictWithId;
    }) => any;
    delete: (arg: {
        select?: SelectWithId | null;
        include?: Dict | null;
        where: DictWithId;
    }) => any;
    [k: string]: any;
}
export declare type FormatErrorHandler = (error: TypeError, logHeader: string) => void;
declare type BrowseOption = {
    scanAll?: boolean;
    sort?: object;
    speedMode?: boolean;
};
export declare class BaseModel<PrismaClient extends BasePrismaClient, CustomCreateInput, WhereUniqueInput, WhereInput, CustomUpdateInput, R = unknown> {
    protected readonly prisma: PrismaClient;
    protected readonly modelName: string;
    protected readonly formatErrorHandler: FormatErrorHandler;
    constructor(prisma: PrismaClient, formatErrorHandler: FormatErrorHandler, modelName?: string);
    add(input: CustomCreateInput, requestedBy: string): Promise<R>;
    read(where: WhereUniqueInput): Promise<R>;
    browse(pagination?: IPagination, where?: WhereInput | BaseWhereInput, options?: BrowseOption): Promise<{
        items: R[];
        properties: {
            count?: number;
            pageSize?: number;
            page?: number;
        };
    }>;
    edit(input: CustomUpdateInput, requestedBy: string): Promise<R>;
    delete(where: WhereUniqueInput): Promise<R>;
    deactivateByUuid(uuid: string, requestedBy: string): Promise<R>;
}
export declare function maskingPrint(json: any): any;
