"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.authDirective = exports.authDirectiveTypeDefs = void 0;
const lodash_1 = __importDefault(require("lodash"));
const apollo_server_core_1 = require("apollo-server-core");
const utils_1 = require("@graphql-tools/utils");
const graphql_1 = require("graphql");
const cm_libs_context_aware_error_1 = require("cm-libs-context-aware-error");
const common_1 = require("../common");
exports.authDirectiveTypeDefs = (0, apollo_server_core_1.gql) `
    directive @auth(type: String) on FIELD | FIELD_DEFINITION
`;
function authDirective(schema, directiveName = 'auth') {
    return (0, utils_1.mapSchema)(schema, {
        [utils_1.MapperKind.OBJECT_FIELD]: (fieldConfig) => {
            const authDirective = (0, utils_1.getDirective)(schema, fieldConfig, directiveName)?.[0];
            if (authDirective) {
                const { resolve = graphql_1.defaultFieldResolver } = fieldConfig;
                const keys = Object.keys(authDirective);
                if (!(1 === keys.length && 'type' === keys[0])) {
                    (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.INVALID_PARAMETER, 'Invalid directive: auth(type), missing argument');
                }
                const spec = {
                    type: authDirective[keys[0]]
                };
                if (!lodash_1.default.isString(spec.type)) {
                    (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.INVALID_PARAMETER, 'Invalid directive: auth(type), type must be a String');
                }
                fieldConfig.resolve = async (source, args, context, info) => {
                    await (0, common_1.auth)(context?.req, spec);
                    return resolve(source, args, context, info);
                };
            }
            return fieldConfig;
        }
    });
}
exports.authDirective = authDirective;
//# sourceMappingURL=auth.directive.js.map