'use strict';
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __exportStar = (this && this.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TcpBase = exports.net = void 0;
const net_1 = __importDefault(require("net"));
exports.net = net_1.default;
const app_base_1 = require("./app-base");
__exportStar(require("net"), exports);
class TcpBase extends app_base_1.AppBase {
    constructor() {
        super(...arguments);
        this.connections = {};
        this._connectionCount = 0;
    }
    async init(overrides) {
        return await super.init(undefined, undefined, overrides);
    }
    async term() {
        for (const clientId of Object.keys(this.connections)) {
            const connection = this.connections[clientId];
            connection.socket?.destroy();
            if (this.removeConnection(clientId)) {
                try {
                    await this.onClose(connection, undefined);
                }
                catch (error) {
                    // Ignore this error
                }
            }
        }
        await super.term();
    }
    async addConnection(socket, clientId, ref, keepAliveInterval, idleTimeout) {
        const connection = {
            clientId,
            socket,
            buffer: Buffer.alloc(0),
            isIdle: true,
            ref
        };
        if (0 < idleTimeout) {
            socket.setTimeout(idleTimeout, () => socket.destroy());
        }
        if (0 < keepAliveInterval) {
            socket.setKeepAlive(true, keepAliveInterval);
        }
        let raisedError;
        socket.on('close', () => {
            return this.onCloseEvent(connection, raisedError);
        });
        /* istanbul ignore next */
        socket.on('error', (error) => {
            raisedError = error;
            connection.socket.destroy();
        });
        socket.on('data', async (data) => {
            return this.onDataEvent(connection, data);
        });
        this.connections[connection.clientId] = connection;
        this._connectionCount++;
        try {
            await this.onConnected(connection);
        }
        catch (error) {
            connection.socket.destroy(error);
        }
        return connection;
    }
    removeConnection(clientId) {
        const connection = this.connections[clientId];
        if (connection) {
            delete this.connections[clientId];
            this._connectionCount--;
            return true;
        }
        return false;
    }
    async onDataEvent(connection, data) {
        try {
            connection.buffer = Buffer.concat([connection.buffer, data]);
            if (connection.isIdle) {
                connection.isIdle = false;
                do {
                    const usedLength = await this.onData(connection, data);
                    if (0 >= usedLength) {
                        break;
                    }
                    connection.buffer = connection.buffer.slice(usedLength);
                    data = undefined;
                } while (0 < connection.buffer.length);
                connection.isIdle = true;
            }
        }
        catch (error) {
            connection.socket.destroy(error);
        }
    }
    async onCloseEvent(connection, raisedError) {
        if (this.removeConnection(connection.clientId)) {
            try {
                await this.onClose(connection, raisedError);
            }
            catch (error) {
                // Ignore this error
            }
        }
    }
    async socketWrite(socket, data) {
        if (!socket.write(data)) {
            return new Promise((resolve, reject) => {
                // Handle the socket buffer backpressure
                socket.once('drain', () => resolve()).once('error', (error) => reject(error));
            });
        }
    }
    get connectionCount() {
        return this._connectionCount;
    }
}
exports.TcpBase = TcpBase;
//# sourceMappingURL=tcp-base.js.map