'use strict';
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __exportStar = (this && this.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TcpClient = void 0;
const cm_libs_context_aware_error_1 = require("cm-libs-context-aware-error");
const tcp_base_1 = require("./tcp-base");
__exportStar(require("./tcp-base"), exports);
class TcpClient extends tcp_base_1.TcpBase {
    async connect(clientId, hostname, port, connectTimeout, ref, keepAliveInterval, idleTimeout, detectEarlyClosure) {
        const socket = new tcp_base_1.net.Socket();
        try {
            await this.onConnect(clientId, hostname, port, connectTimeout);
            await new Promise((resolve, reject) => {
                try {
                    // Setup the timer for connection timeout (the duration of connecting)
                    socket.setTimeout(connectTimeout);
                    socket.once('timeout', () => socket.destroy(new Error('ETIMEDOUT')));
                    socket.once('error', (error) => {
                        return reject((0, cm_libs_context_aware_error_1.createError)(cm_libs_context_aware_error_1.errors.CONNECTION_ERROR, error.message, { ...error }));
                    });
                    let closed = false;
                    socket.once('close', () => {
                        closed = true;
                    });
                    socket.once('connect', async () => {
                        // Cater the misbehavior of istio for newly established connection get disconnected when remote side is not available
                        if (0 < detectEarlyClosure) {
                            await new Promise((resolve) => {
                                if (closed) {
                                    return resolve();
                                }
                                // We introduce some delay to monitor the connection closure before mark connection as connected
                                const timer = setTimeout(() => {
                                    return resolve();
                                }, detectEarlyClosure);
                                socket.once('close', () => {
                                    closed = true;
                                    clearTimeout(timer);
                                    return resolve();
                                });
                            });
                        }
                        socket.removeAllListeners();
                        if (closed) {
                            return reject((0, cm_libs_context_aware_error_1.createError)(cm_libs_context_aware_error_1.errors.CONNECTION_ERROR, 'Connection reset'));
                        }
                        return resolve(socket);
                    });
                    socket.connect(port, hostname);
                }
                catch (error) {
                    return reject((0, cm_libs_context_aware_error_1.createError)(cm_libs_context_aware_error_1.errors.CONNECTION_ERROR, error.message, { ...error }));
                }
            });
            return super.addConnection(socket, clientId, ref, keepAliveInterval, idleTimeout);
        }
        catch (error) {
            await this.raiseConnectionError(socket, clientId, error);
            throw error;
        }
    }
    async raiseConnectionError(socket, clientId, error) {
        const connection = {
            clientId,
            socket,
            buffer: Buffer.alloc(0),
            isIdle: true,
            ref: undefined
        };
        try {
            await this.onClose(connection, error);
        }
        catch (error) {
            // Ignore this error
        }
    }
}
exports.TcpClient = TcpClient;
//# sourceMappingURL=tcp-client.js.map